"use strict";

var _GraphVertex = _interopRequireDefault(require("../../../../data-structures/graph/GraphVertex"));

var _GraphEdge = _interopRequireDefault(require("../../../../data-structures/graph/GraphEdge"));

var _Graph = _interopRequireDefault(require("../../../../data-structures/graph/Graph"));

var _articulationPoints = _interopRequireDefault(require("../articulationPoints"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('articulationPoints', function () {
  it('should find articulation points in simple graph', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC);
    var edgeCD = new _GraphEdge.default(vertexC, vertexD);
    var graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeBC).addEdge(edgeCD);
    var articulationPointsSet = Object.values((0, _articulationPoints.default)(graph));
    expect(articulationPointsSet.length).toBe(2);
    expect(articulationPointsSet[0].getKey()).toBe(vertexC.getKey());
    expect(articulationPointsSet[1].getKey()).toBe(vertexB.getKey());
  });
  it('should find articulation points in simple graph with back edge', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC);
    var edgeCD = new _GraphEdge.default(vertexC, vertexD);
    var edgeAC = new _GraphEdge.default(vertexA, vertexC);
    var graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeAC).addEdge(edgeBC).addEdge(edgeCD);
    var articulationPointsSet = Object.values((0, _articulationPoints.default)(graph));
    expect(articulationPointsSet.length).toBe(1);
    expect(articulationPointsSet[0].getKey()).toBe(vertexC.getKey());
  });
  it('should find articulation points in simple graph with back edge #2', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var vertexE = new _GraphVertex.default('E');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC);
    var edgeCD = new _GraphEdge.default(vertexC, vertexD);
    var edgeAE = new _GraphEdge.default(vertexA, vertexE);
    var edgeCE = new _GraphEdge.default(vertexC, vertexE);
    var graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeAE).addEdge(edgeCE).addEdge(edgeBC).addEdge(edgeCD);
    var articulationPointsSet = Object.values((0, _articulationPoints.default)(graph));
    expect(articulationPointsSet.length).toBe(1);
    expect(articulationPointsSet[0].getKey()).toBe(vertexC.getKey());
  });
  it('should find articulation points in graph', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var vertexE = new _GraphVertex.default('E');
    var vertexF = new _GraphVertex.default('F');
    var vertexG = new _GraphVertex.default('G');
    var vertexH = new _GraphVertex.default('H');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC);
    var edgeAC = new _GraphEdge.default(vertexA, vertexC);
    var edgeCD = new _GraphEdge.default(vertexC, vertexD);
    var edgeDE = new _GraphEdge.default(vertexD, vertexE);
    var edgeEG = new _GraphEdge.default(vertexE, vertexG);
    var edgeEF = new _GraphEdge.default(vertexE, vertexF);
    var edgeGF = new _GraphEdge.default(vertexG, vertexF);
    var edgeFH = new _GraphEdge.default(vertexF, vertexH);
    var graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeBC).addEdge(edgeAC).addEdge(edgeCD).addEdge(edgeDE).addEdge(edgeEG).addEdge(edgeEF).addEdge(edgeGF).addEdge(edgeFH);
    var articulationPointsSet = Object.values((0, _articulationPoints.default)(graph));
    expect(articulationPointsSet.length).toBe(4);
    expect(articulationPointsSet[0].getKey()).toBe(vertexF.getKey());
    expect(articulationPointsSet[1].getKey()).toBe(vertexE.getKey());
    expect(articulationPointsSet[2].getKey()).toBe(vertexD.getKey());
    expect(articulationPointsSet[3].getKey()).toBe(vertexC.getKey());
  });
  it('should find articulation points in graph starting with articulation root vertex', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var vertexE = new _GraphVertex.default('E');
    var vertexF = new _GraphVertex.default('F');
    var vertexG = new _GraphVertex.default('G');
    var vertexH = new _GraphVertex.default('H');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC);
    var edgeAC = new _GraphEdge.default(vertexA, vertexC);
    var edgeCD = new _GraphEdge.default(vertexC, vertexD);
    var edgeDE = new _GraphEdge.default(vertexD, vertexE);
    var edgeEG = new _GraphEdge.default(vertexE, vertexG);
    var edgeEF = new _GraphEdge.default(vertexE, vertexF);
    var edgeGF = new _GraphEdge.default(vertexG, vertexF);
    var edgeFH = new _GraphEdge.default(vertexF, vertexH);
    var graph = new _Graph.default();
    graph.addEdge(edgeDE).addEdge(edgeAB).addEdge(edgeBC).addEdge(edgeAC).addEdge(edgeCD).addEdge(edgeEG).addEdge(edgeEF).addEdge(edgeGF).addEdge(edgeFH);
    var articulationPointsSet = Object.values((0, _articulationPoints.default)(graph));
    expect(articulationPointsSet.length).toBe(4);
    expect(articulationPointsSet[0].getKey()).toBe(vertexF.getKey());
    expect(articulationPointsSet[1].getKey()).toBe(vertexE.getKey());
    expect(articulationPointsSet[2].getKey()).toBe(vertexC.getKey());
    expect(articulationPointsSet[3].getKey()).toBe(vertexD.getKey());
  });
  it('should find articulation points in yet another graph #1', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var vertexE = new _GraphVertex.default('E');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB);
    var edgeAC = new _GraphEdge.default(vertexA, vertexC);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC);
    var edgeCD = new _GraphEdge.default(vertexC, vertexD);
    var edgeDE = new _GraphEdge.default(vertexD, vertexE);
    var graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeAC).addEdge(edgeBC).addEdge(edgeCD).addEdge(edgeDE);
    var articulationPointsSet = Object.values((0, _articulationPoints.default)(graph));
    expect(articulationPointsSet.length).toBe(2);
    expect(articulationPointsSet[0].getKey()).toBe(vertexD.getKey());
    expect(articulationPointsSet[1].getKey()).toBe(vertexC.getKey());
  });
  it('should find articulation points in yet another graph #2', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var vertexE = new _GraphVertex.default('E');
    var vertexF = new _GraphVertex.default('F');
    var vertexG = new _GraphVertex.default('G');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB);
    var edgeAC = new _GraphEdge.default(vertexA, vertexC);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC);
    var edgeCD = new _GraphEdge.default(vertexC, vertexD);
    var edgeCE = new _GraphEdge.default(vertexC, vertexE);
    var edgeCF = new _GraphEdge.default(vertexC, vertexF);
    var edgeEG = new _GraphEdge.default(vertexE, vertexG);
    var edgeFG = new _GraphEdge.default(vertexF, vertexG);
    var graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeAC).addEdge(edgeBC).addEdge(edgeCD).addEdge(edgeCE).addEdge(edgeCF).addEdge(edgeEG).addEdge(edgeFG);
    var articulationPointsSet = Object.values((0, _articulationPoints.default)(graph));
    expect(articulationPointsSet.length).toBe(1);
    expect(articulationPointsSet[0].getKey()).toBe(vertexC.getKey());
  });
});