"use strict";

var _GraphVertex = _interopRequireDefault(require("../../../../data-structures/graph/GraphVertex"));

var _GraphEdge = _interopRequireDefault(require("../../../../data-structures/graph/GraphEdge"));

var _Graph = _interopRequireDefault(require("../../../../data-structures/graph/Graph"));

var _eulerianPath = _interopRequireDefault(require("../eulerianPath"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('eulerianPath', function () {
  it('should throw an error when graph is not Eulerian', function () {
    function findEulerianPathInNotEulerianGraph() {
      var vertexA = new _GraphVertex.default('A');
      var vertexB = new _GraphVertex.default('B');
      var vertexC = new _GraphVertex.default('C');
      var vertexD = new _GraphVertex.default('D');
      var vertexE = new _GraphVertex.default('E');
      var edgeAB = new _GraphEdge.default(vertexA, vertexB);
      var edgeAC = new _GraphEdge.default(vertexA, vertexC);
      var edgeBC = new _GraphEdge.default(vertexB, vertexC);
      var edgeBD = new _GraphEdge.default(vertexB, vertexD);
      var edgeCE = new _GraphEdge.default(vertexC, vertexE);
      var graph = new _Graph.default();
      graph.addEdge(edgeAB).addEdge(edgeAC).addEdge(edgeBC).addEdge(edgeBD).addEdge(edgeCE);
      (0, _eulerianPath.default)(graph);
    }

    expect(findEulerianPathInNotEulerianGraph).toThrowError();
  });
  it('should find Eulerian Circuit in graph', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var vertexE = new _GraphVertex.default('E');
    var vertexF = new _GraphVertex.default('F');
    var vertexG = new _GraphVertex.default('G');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB);
    var edgeAE = new _GraphEdge.default(vertexA, vertexE);
    var edgeAF = new _GraphEdge.default(vertexA, vertexF);
    var edgeAG = new _GraphEdge.default(vertexA, vertexG);
    var edgeGF = new _GraphEdge.default(vertexG, vertexF);
    var edgeBE = new _GraphEdge.default(vertexB, vertexE);
    var edgeEB = new _GraphEdge.default(vertexE, vertexB);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC);
    var edgeED = new _GraphEdge.default(vertexE, vertexD);
    var edgeCD = new _GraphEdge.default(vertexC, vertexD);
    var graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeAE).addEdge(edgeAF).addEdge(edgeAG).addEdge(edgeGF).addEdge(edgeBE).addEdge(edgeEB).addEdge(edgeBC).addEdge(edgeED).addEdge(edgeCD);
    var graphEdgesCount = graph.getAllEdges().length;
    var eulerianPathSet = (0, _eulerianPath.default)(graph);
    expect(eulerianPathSet.length).toBe(graphEdgesCount + 1);
    expect(eulerianPathSet[0].getKey()).toBe(vertexA.getKey());
    expect(eulerianPathSet[1].getKey()).toBe(vertexB.getKey());
    expect(eulerianPathSet[2].getKey()).toBe(vertexE.getKey());
    expect(eulerianPathSet[3].getKey()).toBe(vertexB.getKey());
    expect(eulerianPathSet[4].getKey()).toBe(vertexC.getKey());
    expect(eulerianPathSet[5].getKey()).toBe(vertexD.getKey());
    expect(eulerianPathSet[6].getKey()).toBe(vertexE.getKey());
    expect(eulerianPathSet[7].getKey()).toBe(vertexA.getKey());
    expect(eulerianPathSet[8].getKey()).toBe(vertexF.getKey());
    expect(eulerianPathSet[9].getKey()).toBe(vertexG.getKey());
    expect(eulerianPathSet[10].getKey()).toBe(vertexA.getKey());
  });
  it('should find Eulerian Path in graph', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var vertexE = new _GraphVertex.default('E');
    var vertexF = new _GraphVertex.default('F');
    var vertexG = new _GraphVertex.default('G');
    var vertexH = new _GraphVertex.default('H');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB);
    var edgeAC = new _GraphEdge.default(vertexA, vertexC);
    var edgeBD = new _GraphEdge.default(vertexB, vertexD);
    var edgeDC = new _GraphEdge.default(vertexD, vertexC);
    var edgeCE = new _GraphEdge.default(vertexC, vertexE);
    var edgeEF = new _GraphEdge.default(vertexE, vertexF);
    var edgeFH = new _GraphEdge.default(vertexF, vertexH);
    var edgeFG = new _GraphEdge.default(vertexF, vertexG);
    var edgeHG = new _GraphEdge.default(vertexH, vertexG);
    var graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeAC).addEdge(edgeBD).addEdge(edgeDC).addEdge(edgeCE).addEdge(edgeEF).addEdge(edgeFH).addEdge(edgeFG).addEdge(edgeHG);
    var graphEdgesCount = graph.getAllEdges().length;
    var eulerianPathSet = (0, _eulerianPath.default)(graph);
    expect(eulerianPathSet.length).toBe(graphEdgesCount + 1);
    expect(eulerianPathSet[0].getKey()).toBe(vertexC.getKey());
    expect(eulerianPathSet[1].getKey()).toBe(vertexA.getKey());
    expect(eulerianPathSet[2].getKey()).toBe(vertexB.getKey());
    expect(eulerianPathSet[3].getKey()).toBe(vertexD.getKey());
    expect(eulerianPathSet[4].getKey()).toBe(vertexC.getKey());
    expect(eulerianPathSet[5].getKey()).toBe(vertexE.getKey());
    expect(eulerianPathSet[6].getKey()).toBe(vertexF.getKey());
    expect(eulerianPathSet[7].getKey()).toBe(vertexH.getKey());
    expect(eulerianPathSet[8].getKey()).toBe(vertexG.getKey());
    expect(eulerianPathSet[9].getKey()).toBe(vertexF.getKey());
  });
});