"use strict";

var _GraphVertex = _interopRequireDefault(require("../../../../data-structures/graph/GraphVertex"));

var _GraphEdge = _interopRequireDefault(require("../../../../data-structures/graph/GraphEdge"));

var _Graph = _interopRequireDefault(require("../../../../data-structures/graph/Graph"));

var _prim = _interopRequireDefault(require("../prim"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('prim', function () {
  it('should fire an error for directed graph', function () {
    function applyPrimToDirectedGraph() {
      var graph = new _Graph.default(true);
      (0, _prim.default)(graph);
    }

    expect(applyPrimToDirectedGraph).toThrowError();
  });
  it('should find minimum spanning tree', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var vertexE = new _GraphVertex.default('E');
    var vertexF = new _GraphVertex.default('F');
    var vertexG = new _GraphVertex.default('G');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB, 2);
    var edgeAD = new _GraphEdge.default(vertexA, vertexD, 3);
    var edgeAC = new _GraphEdge.default(vertexA, vertexC, 3);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC, 4);
    var edgeBE = new _GraphEdge.default(vertexB, vertexE, 3);
    var edgeDF = new _GraphEdge.default(vertexD, vertexF, 7);
    var edgeEC = new _GraphEdge.default(vertexE, vertexC, 1);
    var edgeEF = new _GraphEdge.default(vertexE, vertexF, 8);
    var edgeFG = new _GraphEdge.default(vertexF, vertexG, 9);
    var edgeFC = new _GraphEdge.default(vertexF, vertexC, 6);
    var graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeAD).addEdge(edgeAC).addEdge(edgeBC).addEdge(edgeBE).addEdge(edgeDF).addEdge(edgeEC).addEdge(edgeEF).addEdge(edgeFC).addEdge(edgeFG);
    expect(graph.getWeight()).toEqual(46);
    var minimumSpanningTree = (0, _prim.default)(graph);
    expect(minimumSpanningTree.getWeight()).toBe(24);
    expect(minimumSpanningTree.getAllVertices().length).toBe(graph.getAllVertices().length);
    expect(minimumSpanningTree.getAllEdges().length).toBe(graph.getAllVertices().length - 1);
    expect(minimumSpanningTree.toString()).toBe('A,B,C,E,D,F,G');
  });
  it('should find minimum spanning tree for simple graph', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB, 1);
    var edgeAD = new _GraphEdge.default(vertexA, vertexD, 3);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC, 1);
    var edgeBD = new _GraphEdge.default(vertexB, vertexD, 3);
    var edgeCD = new _GraphEdge.default(vertexC, vertexD, 1);
    var graph = new _Graph.default();
    graph.addEdge(edgeAB).addEdge(edgeAD).addEdge(edgeBC).addEdge(edgeBD).addEdge(edgeCD);
    expect(graph.getWeight()).toEqual(9);
    var minimumSpanningTree = (0, _prim.default)(graph);
    expect(minimumSpanningTree.getWeight()).toBe(3);
    expect(minimumSpanningTree.getAllVertices().length).toBe(graph.getAllVertices().length);
    expect(minimumSpanningTree.getAllEdges().length).toBe(graph.getAllVertices().length - 1);
    expect(minimumSpanningTree.toString()).toBe('A,B,C,D');
  });
});