"use strict";

var _GraphVertex = _interopRequireDefault(require("../../../../data-structures/graph/GraphVertex"));

var _GraphEdge = _interopRequireDefault(require("../../../../data-structures/graph/GraphEdge"));

var _Graph = _interopRequireDefault(require("../../../../data-structures/graph/Graph"));

var _stronglyConnectedComponents = _interopRequireDefault(require("../stronglyConnectedComponents"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('stronglyConnectedComponents', function () {
  it('should detect strongly connected components in simple graph', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC);
    var edgeCA = new _GraphEdge.default(vertexC, vertexA);
    var edgeCD = new _GraphEdge.default(vertexC, vertexD);
    var graph = new _Graph.default(true);
    graph.addEdge(edgeAB).addEdge(edgeBC).addEdge(edgeCA).addEdge(edgeCD);
    var components = (0, _stronglyConnectedComponents.default)(graph);
    expect(components).toBeDefined();
    expect(components.length).toBe(2);
    expect(components[0][0].getKey()).toBe(vertexA.getKey());
    expect(components[0][1].getKey()).toBe(vertexC.getKey());
    expect(components[0][2].getKey()).toBe(vertexB.getKey());
    expect(components[1][0].getKey()).toBe(vertexD.getKey());
  });
  it('should detect strongly connected components in graph', function () {
    var vertexA = new _GraphVertex.default('A');
    var vertexB = new _GraphVertex.default('B');
    var vertexC = new _GraphVertex.default('C');
    var vertexD = new _GraphVertex.default('D');
    var vertexE = new _GraphVertex.default('E');
    var vertexF = new _GraphVertex.default('F');
    var vertexG = new _GraphVertex.default('G');
    var vertexH = new _GraphVertex.default('H');
    var vertexI = new _GraphVertex.default('I');
    var vertexJ = new _GraphVertex.default('J');
    var vertexK = new _GraphVertex.default('K');
    var edgeAB = new _GraphEdge.default(vertexA, vertexB);
    var edgeBC = new _GraphEdge.default(vertexB, vertexC);
    var edgeCA = new _GraphEdge.default(vertexC, vertexA);
    var edgeBD = new _GraphEdge.default(vertexB, vertexD);
    var edgeDE = new _GraphEdge.default(vertexD, vertexE);
    var edgeEF = new _GraphEdge.default(vertexE, vertexF);
    var edgeFD = new _GraphEdge.default(vertexF, vertexD);
    var edgeGF = new _GraphEdge.default(vertexG, vertexF);
    var edgeGH = new _GraphEdge.default(vertexG, vertexH);
    var edgeHI = new _GraphEdge.default(vertexH, vertexI);
    var edgeIJ = new _GraphEdge.default(vertexI, vertexJ);
    var edgeJG = new _GraphEdge.default(vertexJ, vertexG);
    var edgeJK = new _GraphEdge.default(vertexJ, vertexK);
    var graph = new _Graph.default(true);
    graph.addEdge(edgeAB).addEdge(edgeBC).addEdge(edgeCA).addEdge(edgeBD).addEdge(edgeDE).addEdge(edgeEF).addEdge(edgeFD).addEdge(edgeGF).addEdge(edgeGH).addEdge(edgeHI).addEdge(edgeIJ).addEdge(edgeJG).addEdge(edgeJK);
    var components = (0, _stronglyConnectedComponents.default)(graph);
    expect(components).toBeDefined();
    expect(components.length).toBe(4);
    expect(components[0][0].getKey()).toBe(vertexG.getKey());
    expect(components[0][1].getKey()).toBe(vertexJ.getKey());
    expect(components[0][2].getKey()).toBe(vertexI.getKey());
    expect(components[0][3].getKey()).toBe(vertexH.getKey());
    expect(components[1][0].getKey()).toBe(vertexK.getKey());
    expect(components[2][0].getKey()).toBe(vertexA.getKey());
    expect(components[2][1].getKey()).toBe(vertexC.getKey());
    expect(components[2][2].getKey()).toBe(vertexB.getKey());
    expect(components[3][0].getKey()).toBe(vertexD.getKey());
    expect(components[3][1].getKey()).toBe(vertexF.getKey());
    expect(components[3][2].getKey()).toBe(vertexE.getKey());
  });
});