"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = binarySearch;

var _Comparator = _interopRequireDefault(require("../../../utils/comparator/Comparator"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Binary search implementation.
 *
 * @param {*[]} sortedArray
 * @param {*} seekElement
 * @param {function(a, b)} [comparatorCallback]
 * @return {number}
 */
function binarySearch(sortedArray, seekElement, comparatorCallback) {
  var comparator = new _Comparator.default(comparatorCallback);
  var startIndex = 0;
  var endIndex = sortedArray.length - 1;

  while (startIndex <= endIndex) {
    var middleIndex = startIndex + Math.floor((endIndex - startIndex) / 2); // If we've found the element just return its position.

    if (comparator.equal(sortedArray[middleIndex], seekElement)) {
      return middleIndex;
    } // Decide which half to choose for seeking next: left or right one.


    if (comparator.lessThan(sortedArray[middleIndex], seekElement)) {
      // Go to the right half of the array.
      startIndex = middleIndex + 1;
    } else {
      // Go to the left half of the array.
      endIndex = middleIndex - 1;
    }
  }

  return -1;
}