"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = btPowerSet;

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

/**
 * @param {*[]} originalSet - Original set of elements we're forming power-set of.
 * @param {*[][]} allSubsets - All subsets that have been formed so far.
 * @param {*[]} currentSubSet - Current subset that we're forming at the moment.
 * @param {number} startAt - The position of in original set we're starting to form current subset.
 * @return {*[][]} - All subsets of original set.
 */
function btPowerSetRecursive(originalSet) {
  var allSubsets = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [[]];
  var currentSubSet = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  var startAt = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;

  // In order to avoid duplication we need to start from next element every time we're forming a
  // subset. If we will start from zero then we'll have duplicates like {3, 3, 3}.
  for (var position = startAt; position < originalSet.length; position += 1) {
    // Let's push current element to the subset.
    currentSubSet.push(originalSet[position]); // Current subset is already valid so let's memorize it.

    allSubsets.push(_toConsumableArray(currentSubSet)); // Let's try to form all other subsets for the current subset.

    btPowerSetRecursive(originalSet, allSubsets, currentSubSet, position + 1); // BACKTRACK. Exclude last element from the subset and try the next one.

    currentSubSet.pop();
  } // Return all subsets of a set.


  return allSubsets;
}
/**
 * Find power-set of a set using BACKTRACKING approach.
 *
 * @param {*[]} originalSet
 * @return {*[][]}
 */


function btPowerSet(originalSet) {
  return btPowerSetRecursive(originalSet);
}