"use strict";

var _BloomFilter = _interopRequireDefault(require("../BloomFilter"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

describe('BloomFilter', function () {
  var bloomFilter;
  var people = ['Bruce Wayne', 'Clark Kent', 'Barry Allen'];
  beforeEach(function () {
    bloomFilter = new _BloomFilter.default();
  });
  it('should have methods named "insert" and "mayContain"', function () {
    expect(_typeof(bloomFilter.insert)).toBe('function');
    expect(_typeof(bloomFilter.mayContain)).toBe('function');
  });
  it('should create a new filter store with the appropriate methods', function () {
    var store = bloomFilter.createStore(18);
    expect(_typeof(store.getValue)).toBe('function');
    expect(_typeof(store.setValue)).toBe('function');
  });
  it('should hash deterministically with all 3 hash functions', function () {
    var str1 = 'apple';
    expect(bloomFilter.hash1(str1)).toEqual(bloomFilter.hash1(str1));
    expect(bloomFilter.hash2(str1)).toEqual(bloomFilter.hash2(str1));
    expect(bloomFilter.hash3(str1)).toEqual(bloomFilter.hash3(str1));
    expect(bloomFilter.hash1(str1)).toBe(14);
    expect(bloomFilter.hash2(str1)).toBe(43);
    expect(bloomFilter.hash3(str1)).toBe(10);
    var str2 = 'orange';
    expect(bloomFilter.hash1(str2)).toEqual(bloomFilter.hash1(str2));
    expect(bloomFilter.hash2(str2)).toEqual(bloomFilter.hash2(str2));
    expect(bloomFilter.hash3(str2)).toEqual(bloomFilter.hash3(str2));
    expect(bloomFilter.hash1(str2)).toBe(0);
    expect(bloomFilter.hash2(str2)).toBe(61);
    expect(bloomFilter.hash3(str2)).toBe(10);
  });
  it('should create an array with 3 hash values', function () {
    expect(bloomFilter.getHashValues('abc').length).toBe(3);
    expect(bloomFilter.getHashValues('abc')).toEqual([66, 63, 54]);
  });
  it('should insert strings correctly and return true when checking for inserted values', function () {
    people.forEach(function (person) {
      return bloomFilter.insert(person);
    });
    expect(bloomFilter.mayContain('Bruce Wayne')).toBe(true);
    expect(bloomFilter.mayContain('Clark Kent')).toBe(true);
    expect(bloomFilter.mayContain('Barry Allen')).toBe(true);
    expect(bloomFilter.mayContain('Tony Stark')).toBe(false);
  });
});