"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _BinaryTreeNode2 = _interopRequireDefault(require("../BinaryTreeNode"));

var _Comparator = _interopRequireDefault(require("../../../utils/comparator/Comparator"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

var BinarySearchTreeNode =
/*#__PURE__*/
function (_BinaryTreeNode) {
  _inherits(BinarySearchTreeNode, _BinaryTreeNode);

  /**
   * @param {*} [value] - node value.
   * @param {function} [compareFunction] - comparator function for node values.
   */
  function BinarySearchTreeNode() {
    var _this;

    var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
    var compareFunction = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;

    _classCallCheck(this, BinarySearchTreeNode);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(BinarySearchTreeNode).call(this, value)); // This comparator is used to compare node values with each other.

    _this.compareFunction = compareFunction;
    _this.nodeValueComparator = new _Comparator.default(compareFunction);
    return _this;
  }
  /**
   * @param {*} value
   * @return {BinarySearchTreeNode}
   */


  _createClass(BinarySearchTreeNode, [{
    key: "insert",
    value: function insert(value) {
      if (this.nodeValueComparator.equal(this.value, null)) {
        this.value = value;
        return this;
      }

      if (this.nodeValueComparator.lessThan(value, this.value)) {
        // Insert to the left.
        if (this.left) {
          return this.left.insert(value);
        }

        var newNode = new BinarySearchTreeNode(value, this.compareFunction);
        this.setLeft(newNode);
        return newNode;
      }

      if (this.nodeValueComparator.greaterThan(value, this.value)) {
        // Insert to the right.
        if (this.right) {
          return this.right.insert(value);
        }

        var _newNode = new BinarySearchTreeNode(value, this.compareFunction);

        this.setRight(_newNode);
        return _newNode;
      }

      return this;
    }
    /**
     * @param {*} value
     * @return {BinarySearchTreeNode}
     */

  }, {
    key: "find",
    value: function find(value) {
      // Check the root.
      if (this.nodeValueComparator.equal(this.value, value)) {
        return this;
      }

      if (this.nodeValueComparator.lessThan(value, this.value) && this.left) {
        // Check left nodes.
        return this.left.find(value);
      }

      if (this.nodeValueComparator.greaterThan(value, this.value) && this.right) {
        // Check right nodes.
        return this.right.find(value);
      }

      return null;
    }
    /**
     * @param {*} value
     * @return {boolean}
     */

  }, {
    key: "contains",
    value: function contains(value) {
      return !!this.find(value);
    }
    /**
     * @param {*} value
     * @return {boolean}
     */

  }, {
    key: "remove",
    value: function remove(value) {
      var nodeToRemove = this.find(value);

      if (!nodeToRemove) {
        throw new Error('Item not found in the tree');
      }

      var parent = nodeToRemove.parent;

      if (!nodeToRemove.left && !nodeToRemove.right) {
        // Node is a leaf and thus has no children.
        if (parent) {
          // Node has a parent. Just remove the pointer to this node from the parent.
          parent.removeChild(nodeToRemove);
        } else {
          // Node has no parent. Just erase current node value.
          nodeToRemove.setValue(undefined);
        }
      } else if (nodeToRemove.left && nodeToRemove.right) {
        // Node has two children.
        // Find the next biggest value (minimum value in the right branch)
        // and replace current value node with that next biggest value.
        var nextBiggerNode = nodeToRemove.right.findMin();

        if (!this.nodeComparator.equal(nextBiggerNode, nodeToRemove.right)) {
          this.remove(nextBiggerNode.value);
          nodeToRemove.setValue(nextBiggerNode.value);
        } else {
          // In case if next right value is the next bigger one and it doesn't have left child
          // then just replace node that is going to be deleted with the right node.
          nodeToRemove.setValue(nodeToRemove.right.value);
          nodeToRemove.setRight(nodeToRemove.right.right);
        }
      } else {
        // Node has only one child.
        // Make this child to be a direct child of current node's parent.

        /** @var BinarySearchTreeNode */
        var childNode = nodeToRemove.left || nodeToRemove.right;

        if (parent) {
          parent.replaceChild(nodeToRemove, childNode);
        } else {
          _BinaryTreeNode2.default.copyNode(childNode, nodeToRemove);
        }
      } // Clear the parent of removed node.


      nodeToRemove.parent = null;
      return true;
    }
    /**
     * @return {BinarySearchTreeNode}
     */

  }, {
    key: "findMin",
    value: function findMin() {
      if (!this.left) {
        return this;
      }

      return this.left.findMin();
    }
  }]);

  return BinarySearchTreeNode;
}(_BinaryTreeNode2.default);

exports.default = BinarySearchTreeNode;