"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _HashTable = _interopRequireDefault(require("../hash-table/HashTable"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var TrieNode =
/*#__PURE__*/
function () {
  /**
   * @param {string} character
   * @param {boolean} isCompleteWord
   */
  function TrieNode(character) {
    var isCompleteWord = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    _classCallCheck(this, TrieNode);

    this.character = character;
    this.isCompleteWord = isCompleteWord;
    this.children = new _HashTable.default();
  }
  /**
   * @param {string} character
   * @return {TrieNode}
   */


  _createClass(TrieNode, [{
    key: "getChild",
    value: function getChild(character) {
      return this.children.get(character);
    }
    /**
     * @param {string} character
     * @param {boolean} isCompleteWord
     * @return {TrieNode}
     */

  }, {
    key: "addChild",
    value: function addChild(character) {
      var isCompleteWord = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      if (!this.children.has(character)) {
        this.children.set(character, new TrieNode(character, isCompleteWord));
      }

      var childNode = this.children.get(character); // In cases similar to adding "car" after "carpet" we need to mark "r" character as complete.

      childNode.isCompleteWord = childNode.isCompleteWord || isCompleteWord;
      return childNode;
    }
    /**
     * @param {string} character
     * @return {TrieNode}
     */

  }, {
    key: "removeChild",
    value: function removeChild(character) {
      var childNode = this.getChild(character); // Delete childNode only if:
      // - childNode has NO children,
      // - childNode.isCompleteWord === false.

      if (childNode && !childNode.isCompleteWord && !childNode.hasChildren()) {
        this.children.delete(character);
      }

      return this;
    }
    /**
     * @param {string} character
     * @return {boolean}
     */

  }, {
    key: "hasChild",
    value: function hasChild(character) {
      return this.children.has(character);
    }
    /**
     * Check whether current TrieNode has children or not.
     * @return {boolean}
     */

  }, {
    key: "hasChildren",
    value: function hasChildren() {
      return this.children.getKeys().length !== 0;
    }
    /**
     * @return {string[]}
     */

  }, {
    key: "suggestChildren",
    value: function suggestChildren() {
      return _toConsumableArray(this.children.getKeys());
    }
    /**
     * @return {string}
     */

  }, {
    key: "toString",
    value: function toString() {
      var childrenAsString = this.suggestChildren().toString();
      childrenAsString = childrenAsString ? ":".concat(childrenAsString) : '';
      var isCompleteString = this.isCompleteWord ? '*' : '';
      return "".concat(this.character).concat(isCompleteString).concat(childrenAsString);
    }
  }]);

  return TrieNode;
}();

exports.default = TrieNode;